module Tickethub
  class Channel < Resource
    path '/channel', singleton: true

    require_relative 'channel/coupon'
    require_relative 'channel/tier'
    require_relative 'channel/variant'
    require_relative 'channel/extra'
    require_relative 'channel/question'
    require_relative 'channel/option'

    require_relative 'channel/booking'
    require_relative 'channel/order'
    require_relative 'channel/payment'

    require_relative 'channel/product'
    require_relative 'channel/supplier'

    collection :coupons, Coupon
    collection :tiers, Tier
    collection :variants, Variant
    collection :extras, Extra
    collection :questions, Question
    collection :options, Option

    collection :products, Product
    collection :bookings, Booking
    collection :orders, Order
    collection :payments, Payment

    association :supplier, Supplier

    def self.[](attributes)
      token = attributes[:token].is_a?(String) ? attributes[:token]
        : attributes[:token][:access_token]
      self.call Tickethub.endpoint(auth_type: :bearer, password: token)[path]
    end

    def initialize(endpoint, attributes = nil)
      attributes ||= endpoint.get

      if attributes['token']
        endpoint = Tickethub.endpoint(auth_type: :bearer,
          password: attributes['token']['access_token'])[self.class.path]
      end

      super(endpoint, attributes)
    end
  end
end